*** Integer math formats ***

* A real in the (open) interval (-1, +1) is
* represented as one byte. Byte values 0 to
* 127 are positive, and values 129 to 255
* are negative. 128 should not be used.
*
* Conversion from this format to reals:
* Positive: x/128             0<=x<=127 
* Negative: -((256-x)/128)  129<=x<=255
*
* Conversion from reals to this format:
* Positive: floor(128*x)         0<=x<1
* Negative: 256-floor(128*(-x))  -1<x<0
*
* All angles are represented as an integer
* between 0 and 255 (a byte) where 256 is
* corresponding to 360. Conversion to
* degrees can be done as (v/256)*360.


*** Data string format ***

CON(5)     =DOSTR
REL(5)     end
CON(4)     alpha & beta
CON(4)     alpha & beta increment
CON(18)    transform matrix (3x3)
CON(2)     number of stars - 1
CON(6)     location of star 1
...
CON(6)     location of star n
CON(1)     hack
CON(34*64) frame 1
CON(34*64) frame 2

* hack is used to assure that the
* frames are aligned on an even
* address so we don't have to worry
* about line offsets.
* Total size: 8+18+2+6*n+1+2*34*64=
* =4381+6n (=#111D+6n)


*** File Main.s ***

* Expects a real between 1 and 255
* on stack. This is the number of
* stars that will be on the boule.
* Allocates a string that will hold
* the frames, stars and other stuff.

::
CK1NOLASTWD
CK&DISPATCH1 real
 ::
 (Make sure real is in valid range)
 DUP %1 %< OVER % 255 %> OR IT
  SETSIZEERR
 COERCE
 (Allocate data string as above)
 NULL$ OVER #6* #111D #+ EXPAND
 SWAP
 CODE
        INCLUDE Main.a
 ENDCODE
 DROP
 FLUSH
 ;
;


*** File Main.a ***

Main    GOSBVL  =POP#
        GOSBVL  =SAVPTR
        GOSUB   Setup
        D0=(5)  =LINECOUNT * Turn of menu
        LC(2)   63         * if it's on
        DAT0=C  B
        INTOFF             * Turn off all
        ST=0    15         * key interrupts
        INCLUDE SwapPage.a
        INCLUDE NewTrans.a
        INCLUDE Draw.a
        INCLUDE CkKeys.a
        GONC    +
        GOTO    SwapPage   * label in SwapPage.a
+       C=0     X
        OUT=C   X          * Don't read keyboard
        INTON              * Turn on keyboard
        ST=1    15         * interrupts again
        INCLUDE OldDisp.a
        GOVLNG  =GETPTRLOOP

        INCLUDE Mul.a
        INCLUDE Dot.a
        INCLUDE Trig.a
        INCLUDE Setup.a


*** File Setup.a ***

Setup   C=DAT1  A          * Store address
        C=C+CON A,10       * of string data 
        R2=C.F  A          * R2[A]
        D1=C
        D1=D1+  8+18       * Store number of
        A=A-1   B          * stars - 1 at correct
        DAT1=A  B          * position in string
        C=A     B
        D=C     B          * Loop counter
        D1=D1+  2          * First star pos
        INCLUDE RPoint.a   * Write a random star
        D=D-1   B
        GOC     +
        GOTO    RPoint     * Label in RPoint.a
+       AD1EX
        ?ABIT=0 0          * Even address?
        GOYES   +
        A=A+1   A          * Use hack
+       R1=A.F  A          * R1[A]->frame 1
        LC(5)   34*64
        A=A+C   A
        R0=A.F  A          * R2[A]->frame 2
        D1=A               * Write info grob
        GOSUB   Info       * to frame 2
        A=R1.F  A          * and to frame 1
        D1=A               * RTN at end of Info.a
        INCLUDE Info.a     * will return from here


*** File OldDisp.a ***

* The method of reading the LINECOUNT
* ghost is from goodies disc #10.

OldDisp GOSBVL  =D0->Row1  * Old screen
        CD0EX
        D0=(5)  =DISP1CTL
        DAT0=C  A
        D0=(5)  (=addrLINECNTg)+2
        C=DAT0  A          * Address of RAM var
        D0=C
        C=DAT0  B          * Old linecount
        D0=(5)  =LINECOUNT
        DAT0=C  B


*** File Draw.a ***

Draw    INCLUDE Blank.a    * Blank out frame
        C=R2.F  A          * String data
        LA(5)   8+18
        C=C+A   A
        D1=C               * Read number of
        A=DAT1  B          * stars - 1
        R3=A.F  A          * Star to draw
        INCLUDE XForm.a
        INCLUDE Plot.a
        A=R3.F  A          * Do loop for
        A=A-1   A          * all stars
        R3=A.F  A
        GOC     +
        GOTO    XForm      * Label in XForm.a
+


*** File NewTrans.a ***

* Updates alpha and beta with increments,
* calculates the following matrix and stores
* it to data string (a for alha, b for beta):
* (  cos(a)  sin(a)*sin(b)  sin(a)*cos(b) )
* (    0        cos(b)         -sin(b)    )
* ( -sin(a)  cos(a)*sin(b)  cos(a)*cos(b) )

NewTrans

        INCLUDE Inc\Ga\Gb.a
        A=0     A
        A=R3.F  B          
        GOSUB   Cos
        DAT1=A  B
        D1=D1+  2
        A=R3.F  B
        GOSUB   Sin
        D0=A               * Save temp result
        A=R4.F  B
        GOSUB   Sin
        CD0EX              * Fetch it
        GOSUB   Mul
        DAT1=A  B
        D1=D1+  2
        A=R3.F  B
        GOSUB   Sin
        D0=A
        A=R4.F  B
        GOSUB   Cos
        CD0EX
        GOSUB   Mul
        DAT1=A  B
        D1=D1+  2
  
        A=0     B
        DAT1=A  B
        D1=D1+  2
        A=R4.F  B
        GOSUB   Cos
        DAT1=A  B
        D1=D1+  2
        A=R4.F  B
        GOSUB   Sin
        A=-A    B
        DAT1=A  B
        D1=D1+  2

        A=R3.F  B
        GOSUB   Sin
        A=-A    B
        DAT1=A  B
        D1=D1+  2
        A=R3.F  B
        GOSUB   Cos
        D0=A
        A=R4.F  B
        GOSUB   Sin
        CD0EX
        GOSUB   Mul
        DAT1=A  B
        D1=D1+  2
        A=R3.F  B
        GOSUB   Cos
        D0=A
        A=R4.F  B
        GOSUB   Cos
        CD0EX
        GOSUB   Mul
        DAT1=A  B


*** File XForm.a ***

XForm   C=R2.F  A          * Point D0 to second
        D0=C               * row of transform
        D0=D0+  8+6        * matrix
        A=R3.F  A
        A=A+A   A          * Multiply current
        C=C+A   A          * star number with 6
        A=A+A   A
        C=C+A   A
        LA(5)   8+18+2     * Skip to first star
        C=C+A   A
        D1=C               * D1-> current star
        GOSUB   Dot        * Transformed y-coord
        GOSUB   ToDisp     * Scale to fit display
        C=R0.F  A          * Add this value
        A=A+A   A          * times 34 to the
        C=C+A   A          * hidden frame address
        ASL     A          * D[A] now points to
        C=C+A   A          * the row to draw the
        D=C     A          * top of the star to
        A=0     A
        D0=D0-  12         * Top row of matrix
        D1=D1-  6          * Back up to this star
        GOSUB   Dot        * Transformed x-coord
        GOSUB   ToDisp     * Scale to fit display
        C=A     A          * Add this value / 4
        CSRB.F  A          * to D to get the exact
        CSRB.F  A          * nibble to draw the
        C=C+D   A          * star to
        R4=C.F  A          * Save it in R[A]
        LCHEX   3          * Mask of the low order
        C=C&A   P          * two bits of x-coord
        D=0     A          * and store it in D[A]
        D=C     P
        D0=D0+  6          * Bottom row of matrix
        D1=D1-  6
        GOSUB   Dot        * Transformed x-coord
        LC(2)   #80        * See ToDisp for an
        A=A+C   B          * explanation
        ASRB.F  A          * Clear out the lowest
        A=0     P          * five bits (A[A] is now
        A=A+A   A          * 0*32 - 7*32)
        GOTO    +

ToDisp  LC(2)   #80        * -1 -> #0, +1 -> #100
        A=A+C   B
        ASL     A          * -1 -> #0, +1 -> #1000
        C=A     A
        A=A+A   A
        A=A+C   A          * -1 -> #0, +1 -> #3000
        ASR     A          * -1 -> #0, +1 -> #300
        ASR     A          * -1 -> #0, +1 -> #30
        C=0     A
        LC(1)   6
        A=A+C   A          * -1 -> #6, +1 -> #36
        RTN
+
  

*** File Plot.a ***

Plot    GOSUB   +
        INCLOB  pics.g
+       C=RSTK
        C=C+A   A          * Go to right row
        D=D+D   A
        C=C+D   A          * Go to right column
        D0=C               * D0-> star image
        C=R4.F  A
        D1=C               * D1-> pos in screen
        LAHEX   3
        B=A     P          * Loop counter
--      A=DAT0  B
        C=DAT1  B
        C=C!A   B          * OR image to screen
        DAT1=C  B
        D0=D0+  8          * Next row of image
        D1=D1+  34
        B=B-1   P
        GONC    --


*** File SwapPage.a ***

SwapPage
 
        D0=(5)  =LINECOUNT * Check screen update
        D0=D0+  1          * Need only high nibble
        LAHEX   3          * Mask
-       C=DAT0  P          * Wait for scan to 
        C=C&A   P          * reach row 16
        ?C=0    P
        GOYES   -
-       C=DAT0  P          * Wait for scan to
        C=C&A   P          * reach row 0 again
        ?C#0    P
        GOYES   -
+       D0=(5)  =DISP1CTL
        C=R0.F  A          * Change display pointer
        DAT0=C  A          * to the hidden frame
        A=R1.F  A
        R0=A.F  A          * Swap frame pointers
        R1=C.F  A          * in R0[A] and R1[A]


*** File CkKeys.a ***

CkKeys  C=R2.F  A
        D1=C
        D1=D1+  6          * D1-> beta increment
        A=DAT1  B
        LCHEX   080        * Scan second row of
        OUT=C   X          * keyboard (up key)
        GOSBVL  =CINRTN
        ?CBIT=0 1          * Was it up?
        GOYES   +
        A=A+1   B          * New increment
+       LCHEX   040        * Scan third row of
        OUT=C   X          * keyboard
        GOSBVL  =CINRTN
        ?CBIT=0 1          * Is down key up?
        GOYES   +
        A=A-1   B          * New increment
+       DAT1=A  B          * Write back
        D1=D1-  2          * D1-> alpha increment
        A=DAT1  B
        ?CBIT=0 2          * Is left key up?
        GOYES   +
        A=A-1   B          * New increment
+       ?CBIT=0 0          * Is right key up?
        GOYES   +
        A=A+1   B          * New increment
+       DAT1=A  B          * Write back
        LCHEX   010        * Scan fifth row
        OUT=C   X          * of keyboard
        GOSBVL  =CINRTN
        LAHEX   10         * If enter was down
        C=C-A   B          * this will set carry


*** File Info.a ***

Info    GOSUB   +
        INCLOB  Info.g
+       C=RSTK
        D0=C               * D0-> info grob
        D1=D1+  16         * D1-> right half
        LC(2)   63         * Loop counter
-       A=DAT0  W
        DAT1=A  W
        D0=D0+  16
        D1=D1+  34
        C=C-1   B
        GONC    -
        RTN


*** File Inc\Ga\Gb.a ***

Inc\Ga\Gb

        C=R2.F  A
        D1=C
        D1=D1+  4
        C=DAT1  B          * Alpha increment
        D1=D1-  4
        A=DAT1  B          * Alpha
        A=A+C   B
        DAT1=A  B
        R3=A.F  B          * New alpha to R3[A]
        D1=D1+  6
        C=DAT1  B          * Beta increment
        D1=D1-  4
        A=DAT1  B          * Beta
        A=A+C   B 
        DAT1=A  B
        R4=A.F  B          * New beta to R4[A]
        D1=D1+  6          * D1-> trans matrix


*** File RPoint.a ***

* This routine calculates random points at
* the unit sphere. To generate angles it uses
* the hardware CRC which is "random" enough
* for this purpose. The CRC changes whenever
* new data appears on the data bus, so we
* read DAT1 too to get more randomness.
* The star gets the following coordinates:
* ( cos(u)  sin(u)*sin(v)  -sin(u)*cos(v) )
* where u and v are "random" angles

RPoint  D0=(5)  =CRC
        A=DAT0  B
        C=DAT1  B
        A=A+C   B
        R4=A.F  B          * Angle v 
        A=DAT0  B
        C=DAT1  B
        A=A+C   B
        R3=A.F  B          * Angle u
        GOSUB   Cos
        DAT1=A  B          * Write x-coord
        D1=D1+  2
        A=R3.F  B
        GOSUB   Sin        * We don't need u
        R3=A.F  B          * now, just sin(u)
        A=R4.F  B
        GOSUB   Sin
        C=R3.F  B
        GOSUB   Mul
        DAT1=A  B          * Write y-coord
        D1=D1+  2
        A=R4.F  B
        GOSUB   Cos
        C=R3.F  B
        GOSUB   Mul
        A=-A    B
        DAT1=A  B          * Write z-coord
        D1=D1+  2          * D1-> next star


*** File Blank.a ***

Blank   C=R0.F  A
        D1=C               * D1-> hidden frame
        A=0     W
        LC(2)   63         * Loop counter
-       DAT1=A  W          * Blank only left
        D1=D1+  34         * half of frame
        C=C-1   B
        GONC    -


*** File Dot.a ***

Dot     A=DAT1  B
        C=DAT0  B
        GOSUB   Mul
        C=A     B          * Accumulate dot
        RSTK=C             * product on stack
        D1=D1+  2
        D0=D0+  2
        A=DAT1  B
        C=DAT0  B
        GOSUB   Mul
        C=RSTK
        C=C+A   B
        RSTK=C
        D1=D1+  2
        D0=D0+  2
        A=DAT1  B
        C=DAT0  B 
        GOSUB   Mul
        C=RSTK
        A=A+C   B
        D1=D1+  2          * Increment pointers
        D0=D0+  2          * past vectors
        RTN


*** File Mul.a ***

* OBS! A[A] must be zero except for A[B]!

Mul     A=0     S          * A[S] = product sign
        ?ABIT=0 7          * A positive?
        GOYES   +
        A=A+1   S
        A=-A    B          * A=abs(A)
+       ?CBIT=0 7          * C positive?
        GOYES   +
        A=A-1   S
        C=-C    B          * C=abs(C)
+       B=0     A          * Product in B[A]
-       ?CBIT=0 0
        GOYES   +
        B=B+A   A
+       CSRB.F  B
        A=A+A   A
        ?C#0    B          * Not done?
        GOYES   -
        A=B     A          * Normalize result
        A=A+A   A          * by shifting A[A]
        ASR     A          * seven bits right
        ASR     A
        ?A=0    S          * Is product positive?
        RTNYES
        A=-A    B          * Negate it
        RTN


*** File Trig.a ***

* Requires A[A] to be zero except for A[B].

Cos     LC(2)   64         * cos(v) = sin(v+90)
        A=A+C   B

Sin     GOSUB   +
        INCLOB  sin.t
+       C=RSTK
        A=A+A   A          * One byte per angle
        C=C+A   A
        CD1EX              * Save D1 in C[A]
        A=0     A          * To be able to call
        A=DAT1  B          * Mul directly
        D1=C               * Restore D1
        RTN


*** File Info.g ***

This file contains a 64 by 64 grob
that will be displayed on the right
side of the screen.


*** File pics.g ***

A 32 by 32 grob composed of 8 by 4
small grobs. The top four grobs is
the image of a star when it is
furthest away, i.e. smallest, and
the bottom row when it is closest.
The second, third and fourth
column are the same as the first
but with the image shifted right
one pixel.


*** File sin.t ***

A 256 character string of sine
values. The first byte holds the
sine of 0, the second sine of
1/256*360 and so on.

